/*
Fit

Copyright (C) 2003 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import java.awt.Label;
import java.awt.event.*;
import java.util.Vector;

import ptolemy.plot.Plot;
import ptolemy.plot.PlotPoint;

public class FitPlot extends Plot implements KeyListener {
    
    double ddx = 50.0;
    double ddy = 5.0;
    Label deltal;
    double sx = 0;
    double sy = 0;
    
    public FitPlot(Label deltal) {
        super();
        this.deltal = deltal;
        addKeyListener(this);
    }
    
    /** Get the data in the specified dataset. This is returned as
     *  a two-dimensional array, where the first index specifies
     *  X or Y data (index 0 or 1 respectively), and the second
     *  index specifies the point.
     *  @return The data in the specified dataset.
     */
    public double[][] getData(int dataset) {
        _checkDatasetIndex(dataset);
        Vector pts = (Vector)_points.elementAt(dataset);
        int size = pts.size();
        double[][] result = new double[2][size];
        for (int i = 0; i < size; i++) {
            PlotPoint pt = (PlotPoint)pts.elementAt(i);
            result[0][i] = pt.x;
            result[1][i] = pt.y;
        }
        return result;
    }

    // Set the data in the specified dataset. The argument is of the
    // form returned by getData.
    public void setData(int dataset, double[][] data) {
        _checkDatasetIndex(dataset);
        Vector pts = (Vector)_points.elementAt(dataset);
        int size = pts.size();
        if (data[0].length < size) size = data[0].length;
        for (int i = 0; i < size; i++) {
            PlotPoint pt = (PlotPoint)pts.elementAt(i);
            pt.x = data[0][i];
            pt.y = data[1][i];
        }
    }
    
    public void keyPressed(KeyEvent e) {
        switch (e.getKeyCode()) {
            case KeyEvent.VK_LEFT: deplacer(-1, 0); break;
            case KeyEvent.VK_RIGHT: deplacer(1, 0); break;
            case KeyEvent.VK_UP: deplacer(0, 1); break;
            case KeyEvent.VK_DOWN: deplacer(0, -1); break;
            default: ;
        }
    }
    
    public void keyReleased(KeyEvent e) {
    }
    
    public void keyTyped(KeyEvent e) {
    }
    
    public void deplacer(int dx, int dy) {
        double[][] data = getData(0);
        for (int i=0; i<data[0].length; i++) {
            data[0][i] += dx/ddx;
            data[1][i] += dy/ddy;
        }
        setData(0, data);
        repaint();
        sx += dx/ddx;
        sy += dy/ddy;
        deltal.setText("d(B-V) : " + Math.round(sx*1000)/1000.0 +
            "    dV : " + Math.round(sy*1000)/1000.0);
    }
    
    void recalcZone() {
        // Recalculate the boundaries based on currently visible data
        _xBottom = Double.MAX_VALUE;
        _xTop = - Double.MIN_VALUE;
        _yBottom = Double.MAX_VALUE;
        _yTop = - Double.MIN_VALUE;
        for (int dataset = 0; dataset < _points.size(); dataset++) {
            Vector points = (Vector)_points.elementAt(dataset);
            for (int index = 0; index < points.size(); index++) {
                PlotPoint pt = (PlotPoint)points.elementAt(index);
                if (pt.x < _xBottom) _xBottom = pt.x;
                if (pt.x > _xTop) _xTop = pt.x;
                if (pt.y < _yBottom) _yBottom = pt.y;
                if (pt.y > _yTop) _yTop = pt.y;
            }
        }
    }
    
}
