/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/

var plot = null; // objet jqplot
var courbe; // points de la courbe ZAMS
var mesures; // points du fichier XML
var nuage; // nuage de points issus du fichier XML, avec correction
var titreGraphe;
var titreMesures;
var inverserY = true;
var titreAxeX = 'B - V';
var titreAxeY = 'V';
var dbmv, dv;
var min_dB_V, max_dB_V;
var min_dV, max_dV;
var curseur_dB_V, curseur_dV;


if (!window.Node) {
    var Node =
    {
        ELEMENT_NODE                :  1,
        ATTRIBUTE_NODE              :  2,
        TEXT_NODE                   :  3,
        CDATA_SECTION_NODE          :  4,
        ENTITY_REFERENCE_NODE       :  5,
        ENTITY_NODE                 :  6,
        PROCESSING_INSTRUCTION_NODE :  7,
        COMMENT_NODE                :  8,
        DOCUMENT_NODE               :  9,
        DOCUMENT_TYPE_NODE          : 10,
        DOCUMENT_FRAGMENT_NODE      : 11,
        NOTATION_NODE               : 12
    };
}

function getParameter(paramName) {
    var searchString = window.location.search.substring(1),
    i, val, params = searchString.split("&");
    
    for (i=0;i<params.length;i++) {
        val = params[i].split("=");
        if (val[0] == paramName) {
            return unescape(val[1]);
        }
    }
    return null;
}

function init() {
    var fichierXML = getParameter('fichierXML');
    var chemin_fichiers = getParameter('chemin_fichiers');
    if (fichierXML == null) {
        alert("Il faut passer un paramètre fichierXML à l'URL");
        return;
    }
    if (chemin_fichiers != null) {
        fichierXML = chemin_fichiers + fichierXML;
    }
    var xmlDoc = chargementXML(fichierXML);
    if (xmlDoc == null) {
        alert('fichier introuvable: ' + fichierXML);
        return;
    }
    var racine = xmlDoc.documentElement;
    if (racine.nodeName != 'plot')
        alert('le fichier XML ne contient pas des données formattées pour Fit (plotml)');
    dbmv = 0;
    dv = 0;
    conversionDonnees(racine);
    calculCourbe();
    calculLimites();
    calculNuage();
    curseur_dB_V = new Curseur('curseur_dB-V', min_dB_V, max_dB_V, dbmv, 121, false, modifCurseur);
    curseur_dV = new Curseur('curseur_dV', min_dV, max_dV, dv, 121, false, modifCurseur);
    affichage();
}

function chargementXML(fichierXML) {
    var xmlhttp = new XMLHttpRequest();
    xmlhttp.open("GET", fichierXML, false);
    try {
        xmlhttp.send();
    } catch (ex) {
        alert(ex.description);
        return(null);
    }
    return(xmlhttp.responseXML);
}

function enfantParNom(parent, nom) {
    for (var n=parent.firstChild; n != null; n = n.nextSibling) {
        if (n.nodeType == Node.ELEMENT_NODE && n.nodeName == nom)
            return(n);
    }
    return(null);
}

function valeurElement(el) {
    if (el == null)
        return(null);
    var f = el.firstChild;
    if (f == null)
        return(null);
    return(f.nodeValue);
}

function arrondi(x) {
    return(Math.round(x*1000) / 1000.0);
}

function conversionDonnees(racine) {
    var title, xLabel, yLabel, dataset, n, x, y;
    title = enfantParNom(racine, 'title')
    if (title != null)
        titreGraphe = valeurElement(title);
    xLabel = enfantParNom(racine, 'xLabel');
    if (xLabel != null)
        titreAxeX = valeurElement(xLabel);
    yLabel = enfantParNom(racine, 'yLabel');
    if (yLabel != null)
        titreAxeY = valeurElement(yLabel);
    dataset = enfantParNom(racine, 'dataset');
    titreMesures = dataset.getAttribute('name');
    mesures = [];
    for (n=dataset.firstChild; n != null; n = n.nextSibling) {
        if (n.nodeType == Node.ELEMENT_NODE && n.nodeName == 'p') {
            x = Number(n.getAttribute('x'));
            y = Number(n.getAttribute('y'));
            if (inverserY)
                y = -y;
            mesures.push([x, y]);
        }
    }
}

function calculLimites() {
    var min_courbe_x, max_courbe_x, min_courbe_y, max_courbe_y, min_nuage_x, max_nuage_x, min_nuage_y, max_nuage_y,
        pt, y, i;
    min_courbe_x = Number.MAX_VALUE;
    max_courbe_x = -Number.MAX_VALUE;
    min_courbe_y = Number.MAX_VALUE;
    max_courbe_y = -Number.MAX_VALUE;
    for (i=0; i<courbe.length; i++) {
        pt = courbe[i];
        if (pt[0] < min_courbe_x)
            min_courbe_x = pt[0];
        if (pt[0] > max_courbe_x)
            max_courbe_x = pt[0];
        if (pt[1] < min_courbe_y)
            min_courbe_y = pt[1];
        if (pt[1] > max_courbe_y)
            max_courbe_y = pt[1];
    }
    min_nuage_x = Number.MAX_VALUE;
    max_nuage_x = -Number.MAX_VALUE;
    min_nuage_y = Number.MAX_VALUE;
    max_nuage_y = -Number.MAX_VALUE;
    for (i=0; i<mesures.length; i++) {
        pt = mesures[i];
        y = pt[1];
        if (inverserY)
            y = -y;
        if (pt[0] < min_nuage_x)
            min_nuage_x = pt[0];
        if (pt[0] > max_nuage_x)
            max_nuage_x = pt[0];
        if (y < min_nuage_y)
            min_nuage_y = y;
        if (y > max_nuage_y)
            max_nuage_y = y;
    }
    min_dB_V = min_courbe_x - max_nuage_x;
    if (min_dB_V > 0)
        min_dB_V = 0;
    min_dB_V = Math.floor(min_dB_V);
    max_dB_V = max_courbe_x - min_nuage_x;
    if (max_dB_V < 0)
        max_dB_V = 0;
    max_dB_V = Math.ceil(max_dB_V);
    min_dV = min_nuage_y - max_courbe_y;
    if (min_dV > 0)
        min_dV = 0;
    min_dV = Math.floor(min_dV);
    max_dV = max_nuage_y - min_courbe_y;
    if (max_dV < 0)
        max_dV = 0;
    max_dV = Math.ceil(max_dV);
}

function calculCourbe() {
    courbe = [];
    // ZAMS en (B-V)0 de -0,4 à 1,4 (Mv de -4,0 à 7,0)
    var x1 = -0.4;
    var x2 = 1.4;
    var npts = 50;
    for (var i=0; i<npts; i++) {
        var x = x1 + ((x2-x1)*i)/(npts - 1);
        var y = 4.900956
            + 3.963022 * (x/0.6-1)
            - 1.945610 * Math.pow((x/0.6-1), 2)
            - 2.387670 * Math.pow((x/0.6-1), 3)
            + 4.926864 * Math.pow((x/0.6-1), 4)
            + 1.852996 * Math.pow((x/0.6-1), 5)
            - 5.092483 * Math.pow((x/0.6-1), 6)
            - 0.391999 * Math.pow((x/0.6-1), 7)
            + 2.084633 * Math.pow((x/0.6-1), 8)
            + 0.019847 * Math.pow((x/0.6-1), 9)
            - 0.315203 * Math.pow((x/0.6-1),10);
        if (inverserY)
            y = -y;
        courbe.push([x, y]);
    }
}

function calculNuage() {
    var x, y, i;
    nuage = [];
    for (i=0; i<mesures.length; i++) {
        x = mesures[i][0] + dbmv;
        y = mesures[i][1] + dv;
        nuage.push([x, y]);
    }
}

function affichage() {
    var span;
    span = document.getElementById('valeur_dB-V');
    setLabelText(span, 'dB-V = ' + arrondi(dbmv));
    span = document.getElementById('valeur_dV');
    setLabelText(span, 'dV = ' + arrondi(dv));
    
    if (plot != null) {
        plot.series[1].data = nuage;
        plot.drawSeries({}, 1);
        return;
    }
    plot = $.jqplot ('plot', [courbe, nuage], {
        title: titreGraphe,
        axes: {
            xaxis: {
                label: titreAxeX
            },
            yaxis: {
                label: titreAxeY,
                tickOptions: {
                    formatter: formatValeurInverse
                }
            }
        },
        seriesDefaults: {
            lineWidth: 1.5,
            shadow: false
        },
        series : [
            {
                label: 'ZAMS',
                showLine: true,
                markerOptions: { show: false }
            },
            {
                label: titreMesures,
                showLine: false,
                markerOptions: { size: 4, shadow: false }
            }
        ],
        legend: {
            show: true
        },
        cursor: {
            show: true,
            zoom: true,
            showTooltip: true
        }
    });
}

function formatValeurInverse(format, val) {
    if (!format)
        format = '%f';
    return($.jqplot.sprintf(format, inverserY ? -val : val));
}

function setLabelText(label, texte) {
    var first = label.firstChild;
    if (first == null) {
        label.appendChild(document.createTextNode(texte));
        return;
    } else
        first.nodeValue = texte;
}

function modifCurseur(e) {
    var id;
    id = e.target.id;
    if (id == 'curseur_dB-V') {
        dbmv = curseur_dB_V.getValeur();
    } else if (id == 'curseur_dV') {
        dv = curseur_dV.getValeur();
    }
    calculNuage();
    affichage();
}
